<?php
  #################################################
  # podblogger 0.5                                #
  # author:      Christian Rotzoll                #
  # co author:   Hagen Toennies                   #
  #                                               #
  # christian.rotzoll@medien.uni-weimar.de        #
  # Doku unter http://www.rotzoll.de/podblogger   #
  #                                               #
  #################################################
  # logFile.class.php                             #
  #################################################

  // HINT: For future work, the logfiles need to be more compressed because they getting large on heavy use

// Define the folder where the logfiles are stored
define("LOGVERZEICHNIS","podbloggerData/stats/");
/**
*   LOGFILE
*  a object to count all audiofile or RSS requests
*  the counting information will be stored in .log files in the
* ."podbloggerData/stats/ folder"
*/
class LogFile {

/**
* Member variables
*
*   // ID of the Object that should be logged
*  var $file_id;
*  // Name of the Object that should be logged
*  var $file_name;
*  // Array of the logging data from the last two months
*  var $array_actualLogs;
*  // Array with logging older than two months
*  var $array_oldLogs;
*/

  // ID of the Object that should be logged
  var $file_id;
  // Name of the Object that should be logged
  var $file_name;
  // Array of the logging data from the last two months
  var $array_actualLogs;
  // Array with logging older than two months
  var $array_oldLogs;

  /**
  * Name: LogFile($inputfile)
  *
  * Constructur builds a Logfile Object
  * loads data from the spezial Logfiles
  */
  function LogFile($inputfile)
  {
    //put the pfath of the folder into the files name
    $this->file_id = str_replace("/","___v___",$inputfile);

    // initialize other  variables
    $this->file_name = $inputfile;
    $this->array_actualLogs = array();
    $this->array_oldLogs = array();

    //load data as log file
    if (file_exists(LOGVERZEICHNIS.$this->file_id.".log"))
    {
      $line_array = file(LOGVERZEICHNIS.$this->file_id.".log");
      $lineCount = 1;

      foreach ($line_array as $line)
      {
        $line = chop($line);
        if ($lineCount == 1)
        {
          // The first lines hold all informations older then two months
          // therefor a spezial handling for the first line
          $this->array_oldLogs = unserialize($line);
        }
        else
        {
          // the next lines holds the information of the last two month
          // one access per line
          $single_log = unserialize($line);
          if (@isset($single_log['UnixTime']))
          {
            //if the logging entrie is correct, put the value into  $array_actualLogs
            $this->array_actualLogs[$single_log['UnixTime']] = $single_log;
          }
        }
        $lineCount++;
      } // endforeach

    } // endif file_exists
  }
/**
* Name: incOldLogLine($type = "ALL")
*
* include the older Logdata
*
*/
  function incOldLogLine($type = "ALL")
  {
    if (!isset($this->array_oldLogs[$type]))
    { $this->array_oldLogs[$type] = 0; }
    $this->array_oldLogs[$type]++;
  }
/**
* Name: igetOldLogLineArray()
*
* return $this->array_oldLogs;
*/
  function getOldLogLineArray()
  {
    return $this->array_oldLogs;
  }

  // Gibt die Anzahl aller Zugriffe auf diese Datei aus
  // $type : ermglicht, dass nur Zugriffe vom Wert $type gezhlt werden
  //	     z.B. $type = "FLASHPLAYER" zhlt nur die Zugriffe, die mit dem Paramerer "FLASHPLAYER" geloggt wurden
  //		 ALL -> Alle Zugriffe
/**
* Name: zugriffeInsgesamt($type = "ALL")
*
* returns the number of all request to the file.
* type defines which kind of request e.g "flashplayer|downloadButton|RSS"
* return $zugriffe;
*/
function zugriffeInsgesamt($type = "ALL")
  {
    $zugriffe = 0;

    //count older requests
    if (isset($this->array_oldLogs[$type]))
    {
      $zugriffe = $zugriffe +  $this->array_oldLogs[$type];
    }

    //count actuell requests
    if ($type == "ALL")
    {
      // count All Requests
       $zugriffe = $zugriffe + count($this->array_actualLogs);
    }
    else
    {
      // count only requests for a given Type
      // therefor iterate all entries

      foreach ($this->array_actualLogs as $log)
      {
        if (isset($log['Type']))
        {
          if ($log['Type'] == $type) {$zugriffe++;}
        }
      }

    }

    return $zugriffe;
  }

  /**
  * Name:zugriffeVonTimeBisTime($startTime, $endTime, $type)
  *
  * gives the count of the file requests between the $startTime $endTime
  * $startTime	: (int)   UnixTime; start of the time period max 2 months ago
  * $endTime	: (int)   UnixTime; end of the time period
  * $type       : (String) type of the counter e.g."flashplayer|downloadButton|RSS"
  */
  function zugriffeVonTimeBisTime($startTime, $endTime, $type)
  {
    $zugriffe = 0;
    // only the counts from the $array_actualLogs will be used
    if ($endTime > time()) {$endTime = time();}

    foreach ($this->array_actualLogs as $log)
    {
      if ( ($log['UnixTime'] >= $startTime) && ($log['UnixTime'] <= $endTime) )
      {
        //only counts between the time Period

        if ($type != "ALL")
        {
          // test Logentrie type
          if (isset($log['Type']))
          {
            if ($log['Type'] == $type) {$zugriffe++;}
          }
        }
        else
        {
          // count Logentrie without type
          $zugriffe++;
        }

      } // endif Zeitfenstercheck
    }  // endforeach

    return $zugriffe;
  } // endfunction

 /**
 * Name: loglinesVonTimeBisTime($startTime, $endTime)
 *
 * $startTime	: (int)   UnixTime; start of the time period max 2 months ago
 * $endTime	: (int)   UnixTime; end of the time period
 */
  function loglinesVonTimeBisTime($startTime, $endTime)
  {
    $erg_array = array();

    //Only count the requests in the $array_actualLogs
    if ($endTime > time()) {$endTime = time();}

    foreach ($this->array_actualLogs as $log)
    {
      if ( ($log['UnixTime'] >= $startTime) && ($log['UnixTime'] <= $endTime) )
      {
        // only counts between the time Period
        array_push($erg_array,$log);
      } // endif
    }  // endforeach

    return $erg_array;
  }
/**
* Name: zugriffeHeute($type = "ALL")
*
* delivers all requests from today
*
*/
function zugriffeHeute($type = "ALL")
  {
    //calculate the Unixtime when the day starts
    $day 	= date("j",time());
    $month	= date("n",time());
    $year	= date("Y",time());

    $Tagesbeginn = mktime(0,0,0,$month,$day,$year);

    return $this->zugriffeVonTimeBisTime($Tagesbeginn, time(), $type);
  }
/**
* Name:  zugriffeGestern($type = "ALL")
*
* delivers all requests from yesterday
*/
function zugriffeGestern($type = "ALL")
  {
    // calculate the Unixtime when the day starts = End of the time period
    $day 	= date("j",time());
    $month	= date("n",time());
    $year	= date("Y",time());

    $GesternEnde = mktime(0,0,0,$month,$day,$year);

    // Unixtime yesterday
    $GesternAnfang = $GesternEnde - (24 * 60 * 60);

    return $this->zugriffeVonTimeBisTime($GesternAnfang, $GesternEnde, $type);
  }

/**
* Name:zugriffeDieserMonat($type = "ALL")
*
* delivers all requests of the month
*/
function zugriffeDieserMonat($type = "ALL")
  {
    //Unixtime from the month starttime
    $month	= date("n",time());
    $year	= date("Y",time());

    $Tagesbeginn = mktime(0,0,0,$month,1,$year);

    return $this->zugriffeVonTimeBisTime($Tagesbeginn, time(), $type);
  }
/**
*Name:  zugriffeLetzterMonat($type = "ALL")
*
* delivers all requests from the last month
*
*/

  function zugriffeLetzterMonat($type = "ALL")
  {
    // Unixtime of the month startime == end of the time period
    $month	= date("n",time());
    $year	= date("Y",time());

    $ZeitfensterEnde = mktime(0,0,0,$month,1,$year);

    // Unixtime of the last month startime = start of the time period
    $month--; if ($month <= 0) {$month=12; $year--;}
    $zeitfensterAnfang = mktime(0,0,0,$month,1,$year);

    return $this->zugriffeVonTimeBisTime($zeitfensterAnfang, $ZeitfensterEnde, $type);
  }
/**
*Name:  zugriffeLetzterMonat($type = "ALL")
*
* delivers all requests older than two months
*
*/
function zugriffeAelterAlsZweiMonate()
  {
    // Unixtime of the last month startime = start of the time period
    $month	= date("n",time());
    $year	= date("Y",time());
    $month--; if ($month <= 0) {$month=12; $year--;}
    $stichtagzeit = mktime(0,0,0,$month,1,$year);

    return $this->zugriffeVonTimeBisTime(0, $stichtagzeit, "ALL");
  }
/**
*Name:  zugriffeLetzterMonat($type = "ALL")
*
* delivers an array with the loglines older than  two months
*
*/
function loglinesAelterAlsZweiMonate()
  {
    // Unixtime of the month startime == end of the time period
    $month	= date("n",time());
    $year	= date("Y",time());
    $month--; if ($month <= 0) {$month=12; $year--;}
    $stichtagzeit = mktime(0,0,0,$month,1,$year);

    return $this->loglinesVonTimeBisTime(0, $stichtagzeit);
  }
/**
*Name:  zugriffeLetzterMonat($type = "ALL")
*
* delivers an array with the loglines of the last two months
*
*/
 function loglinesJuengerAlsZweiMonate()
  {
   // Unixtime of the last month startime = start of the time period
    $month	= date("n",time());
    $year	= date("Y",time());
    $month--; if ($month <= 0) {$month=12; $year--;}
    $stichtagzeit = mktime(0,0,0,$month,1,$year);

    return $this->loglinesVonTimeBisTime($stichtagzeit,time());
  }

 // call the following function like that LogFile::Funktionsname do not instanciate the object

/**
* Name: writeLog($datei, $ip, $type)
*
* writes a log file
*/
function writeLog($datei, $ip, $type)
  {
  //put the pfath of the folder into the files name
  $id = str_replace("/","___v___",$datei);


  // check if a log file, musst be written
  if (!file_exists(LOGVERZEICHNIS.$id.".log"))
  {
    //  prime the Log file
   if ($fp = @fopen(LOGVERZEICHNIS.$id.".log","a"))
   {
     // prime the file with Init-Compressed-line
     $oldLogs = array();
     $oldLogs['ALL'] = 0;
     $logline = serialize($oldLogs);
     @fwrite($fp,$logline."\r\n");
     @fclose($fp);
   }
  }



  if ($fp = @fopen(LOGVERZEICHNIS.$id.".log","a"))
  {
    // create Data array with the log entries
    $log_array  = array();
    $log_array['UnixTime'] 	= time();
    $log_array['Date'] 		= date("d.m.Y H:i:s");
    $log_array['IP']		= $ip;
    $log_array['Type']		= $type;

    $log_line = serialize($log_array);

    @fwrite($fp,$log_line."\r\n");
    @fclose($fp);
  }
 }
/**
* Name: getAllLogFileNames()
*
* delivers an array with the name off all files that have logfiles
*
*/
function getAllLogFileNames()
 {
    $logArray = array();

    $verzeichnis = opendir(LOGVERZEICHNIS);
    while ($datei = readdir($verzeichnis))
    {
     $datNameTeile = explode(".",$datei);
     $endung = array_pop($datNameTeile);
     if (($endung == "log"))
     {
       array_push($logArray, str_replace("___v___","/",implode(".",$datNameTeile)));
     }
    }
    closedir($verzeichnis);

    return $logArray;
  }

/**
* Name: compressOldLogs()
*
* funktion compresses the informations from the last two months into one line
*
*
*/
  function compressOldLogs()
  {
    //find all .log files in the folder
    $logDatnames = LogFile::getAllLogFileNames();

    foreach ($logDatnames as $logDatname)
    {
      $log = new LogFile($logDatname);

      $old_oldLogs = $log->getOldLogLineArray();

      if ($log->zugriffeAelterAlsZweiMonate() > 0)
      {
        //print ("COMPRESS LOGGINGFILE: ".$logDatname."<br>");
        //print (serialize($old_oldLogs)."<br>");
        // here are logs that can be compressed
        $new_actualLogs = $log->loglinesJuengerAlsZweiMonate();
        $old_actualLogs = $log->loglinesAelterAlsZweiMonate();

        //put all actual-Loglines into one line  Alle alten actual-Loglines in die Kompromierte Zeile einzhlen
        foreach ($old_actualLogs as $old_logline)
        {
          // increment the sum
          $log->incOldLogLine();
          // increment the TYP
          if (isset($old_logline['Type']))
          {
            $log->incOldLogLine($old_logline['Type']);
          }
        }

        $new_oldLogs = $log->getOldLogLineArray();
        //print (serialize($new_oldLogs)."<br>");

        // delete old logfiles and write new
        $convert_name =  str_replace("/","___v___",$logDatname.".log");
        @unlink(LOGVERZEICHNIS.$convert_name);

        if ($fp = fopen(LOGVERZEICHNIS.$convert_name,"a"))
        {
          // write compressed line
          fwrite($fp,serialize($new_oldLogs)."\r\n");

          // write the ohter loglines
          foreach ($new_actualLogs as $alog)
          {
            fwrite($fp,serialize($alog)."\r\n");
          }

          fclose($fp);
        }
      }

      // free memory
      unset($log, $new_actualLogs, $old_actualLogs, $new_oldLogs);

    } // end of foreach
  }

} // endclass

?>