<?
  #################################################
  # podblogger 0.5                                #
  # author:      Christian Rotzoll                #
  # co author:   Hagen Toennies                   #
  #                                               #
  # christian.rotzoll@medien.uni-weimar.de        #
  # Doku unter http://www.rotzoll.de/podblogger   #
  #                                               #
  #################################################
  # article.class.bak.php                         #
  #################################################

// generate a random number
mt_srand((double)microtime()*1000000);

################################################################################
# This object implements a data container named "<id>.article" or article file
# for  podblogger contributions

$MAX_COMMENTOBJEKTS = 100;

## Datencontainer, fr einen Artikel (Audio-Beitrag) in podblogger

include("include/comment.class.php");

class article {

  /**
  * Member variables
  * var $id 				= 0;
  * var $unixTime 			= 0;
  * var $file 				= "";
  * var $title 				= "";
  * var $description 			= "";
  * var $authorName 			= "";
  * var $authorMail 			= "";
  * var $authorXtra 			= "";
  * var $authorComment 			= "";
  * var $category 			= "";
  * var $alternative_files	= array();
  * var $commentObjects 	= array();
  * var $objectState 	  	= "INIT";
  *
  */

  var $id 					= 0;
  var $unixTime 			= 0;
  var $file 				= ""; // wenn in file "http://..." oder "ftp://..." liegt, dann nur Internetreferenz
  var $title 				= "";
  var $description 			= "";
  var $authorName 			= "";
  var $authorMail 			= "";
  var $authorXtra 			= "";
  var $authorComment 		= "";
  var $category 			= "";
  var $artwork				= "";

  // two dimensional array ([FORMAT][QUALITY])to hold more than one Audiofile
  // with one article file
  var $alternative_files	= array();

  // every comment to a podblogger contribution is an object
  // every comment will be saved in th $commentObjects variable
  var $commentObjects 		= array();

  // Array mit Datenfeldern zum Rechtemanagement
  var $rightsData 			= array();

  // Array mit Datenfeldern fr erweiterte Funktionen
  var $extendedData         = array();

  // Gibt Auskunft, ob dieses Objekt schon VALID ist
  var $objectState 	  		= "INIT";

/**
* Name: article($id)
*
* constructor function id should always be a unique number.
*/
  function article($id)
  {
    $this->id = $id;
  }

/**
* Name: setUnixTime($data)
*
* set the Unixtime
* $this->unixTime = $data;
*/
  function setUnixTime($data)
  {
    if (!is_numeric($data)) {return FALSE;}
    $this->unixTime = $data;
    $this->checkValid();
  }

/**
* Name: setTitle($data)
*
* set the title of a podblogger contribution
* $this->title = $data;
*/
  function setTitle($data)
  {
    $this->title = $data;
    $this->checkValid();
  }

/**
* Name: setFile($filename, $format="auto", $quality="NORM")
*
* Sets the article file
* $this->file = $data;
* missing ...
*/
  function setFile($filename, $format="auto", $quality="NORM")
  {
    // ggf. automatisch das Dateiformat bestimmen
    if ($format=="auto") {$format = $this->getFormatByFileending($filename);}
    $format = strtoupper($format);

    // prfen, ob bereits eine Mediendatei dem Artikel zugeordnet wurde
    // die erste (das Original wird immer unter $file gespeichert)
    if ($this->file=="") {
      $this->file = $filename;
    }
    else
    {
      // prfen, ob die daten in $file noch in alternative_files kopiert werden
      // mssen (update des datensatzes aus alter version)
      if (!isset($this->alternative_files[$this->getFormatByFileending($this->file)]["NORM"]))
      {
        $this->alternative_files[$this->getFormatByFileending($this->file)]["NORM"] = $this->file;
      }
    }

    // Die Datei in alternative_files vermerken
    $this->alternative_files[$format][$quality]=$filename;

  }

/**
* Name: setDescription($data)
*
* Sets description text for a podblogger contribution
* $this->description = $data;
*/
  function setDescription($data)
  {
    $this->description = $data;
  }

/**
* Name: setAuthorMail($data)
*
* Sets the AuthorMail for a podblogger contribution
* $this->authorMail = $data;
*/
  function setAuthorName($data)
  {
    $this->authorName = $data;
  }


/**
* Name: setAuthorMail($data)
*
* Sets the AuthorMail for a podblogger contribution
* $this->authorMail = $data;
*/
  function setAuthorMail($data)
  {
    $this->authorMail = $data;
  }

/**
* Name: setAuthorXtra($data)
*
* Sets the Author extra information for a podblogger contribution
* $this->authorXtra = $data;
*/
  function setAuthorXtra($data)
  {
    $this->authorXtra = $data;
  }

/**
* Name: setAuthorComment($data)
*
* Sets the Author Comment for a podblogger contribution
* $this->authorComment = $data;
*/
  function setAuthorComment($data)
  {
    $this->authorComment = $data;
  }

/**
* Name: setCategory($data)
*
* Sets the Category  for a podblogger contribution
* $this->category = $data;
*/
  function setCategory($data)
  {
    $this->category = $data;
  }


  function setRightsData($id,$data)
  {
    if (!isset($this->rightsData)) $this->rightsData=array();
    $this->rightsData[$id]=$data;
  }

  function setExtendedData($id,$data)
  {
    if (!isset($this->extendedData)) $this->extendedData=array();
    $this->extendedData[$id]=$data;
  }

  // fgt dem Artikel ein Kommentar-Objekt hinzu
  function addComment($commentobject)
  {
    if ($this->gotComments()<=$GLOBALS['MAX_COMMENTOBJEKTS'])
    {
      $add_it=TRUE;

      // Schutz fr Doppeleintrag von Trackbacks
      if ($commentobject->getType()=="TRACKBACK")
      {
        $testcomments = $this->getComments("TRACKBACK");
        foreach ($testcomments as $tcomment)
        {
          if ($tcomment->getURL()==$commentobject->getURL())
          {$add_it = FALSE;}
        }
      }

      if ($add_it)
      {
        array_push($this->commentObjects, $commentobject);
      }
      return $add_it;
    }
    else
    {
      return FALSE;
    }
  }

  // lscht ein Kommentarobjeckt vom Artkel
  // zur Bestimmung des Kommentars (commentID) wird die Unixtime genutzt
  function deleteComment($commentID)
  {
    if (!isset($this->commentObjects)) {$this->commentObjects = array(); return TRUE;}

    // das betreffende Objekt herausfiltern
    $newObjects = array();
    foreach ($this->commentObjects as $obj)
    {
      if ($obj->getPublishedUnixtime()!=$commentID)
      {
        array_push($newObjects , $obj);
      }
    }
    $this->commentObjects = $newObjects;
    return TRUE;
  }
  // ------------------------------------------------------------
  // Getter-Funktionen
  // ------------------------------------------------------------

/**
* Name: getUnixTime()
*
* return $this->unixTime;
*/
  function getUnixTime()
  {
    if (!isset($this->unixTime)) {$this->unixTime = time();}
    return $this->unixTime;
  }

/**
* Name: getTitle()
*
* returns the title of a podblogger contribution
* return $this->title;
*/
  function getTitle()
  {
    if (!isset($this->title)) {$this->title = "Without a title";}
    return $this->title;
  }

/**
* Name: getFile()
*
* returns the article filr of a podblogger contribution
* return $this->file;
*/
  function getFile()
  {
    if (!isset($this->file)) {$this->file = "";}
    if (isset($this->alternative_files["MP3"]["NORM"])) {return $this->alternative_files["MP3"]["NORM"];}
    return $this->file;
  }

/**
* Name: getDescription()
*
* returns the article description of a podblogger contribution
* return $this->description;
*/
  function getDescription()
  {
    if (!isset($this->description)) {$this->description = "";}
    return $this->description;
  }

/**
* Name: getAuthorName()
*
* returns the authorName of a podblogger contribution
* return $this->authorName;
*/
  function getAuthorName()
  {
    if (!isset($this->authorName)) {$this->authorName = "";}
    return $this->authorName;
  }

/**
* Name: getAuthorMail()
*
* returns the AuthorMail of a podblogger contribution
* return $this->authorMail;
*/
  function getAuthorMail()
  {
    if (!isset($this->authorMail)) {$this->authorMail = "";}
    return $this->authorMail;
  }

/**
* Name: getAuthorXtra()
*
* returns the Authors xtra information of a podblogger contribution
* return $this->authorXtra;
*/
  function getAuthorXtra()
  {
    if (!isset($this->authorXtra)) {$this->authorXtra = "";}
    return $this->authorXtra;
  }

/**
* Name: getAuthorComment()
*
* returns the AuthorComment of a podblogger contribution
* return $this->authorComment;
*/
  function getAuthorComment()
  {
    if (!isset($this->authorComment)) {$this->authorComment = "";}
    return $this->authorComment;
  }

/**
* Name: getCategory()
*
* returns the Category of a podblogger contribution
* return $this->category;
*/
  function getCategory()
  {
    if (!isset($this->category)) {$this->category = "";}
    return $this->category;
  }

/**
* Name: getState()
*
* returns the State of a podblogger contribution
* return $this->objectState;
*/
  function getState()
  {
    return $this->objectState;
  }

/**
* Name: getID()
*
* returns the ID of a podblogger contribution
* return $this->id;
*/
  function getID()
  {
    return $this->id;
  }

  function getRightsData($id)
  {
    if (!isset($this->rightsData)) $this->rightsData=array();
    if (!isset($this->rightsData[$id])) return FALSE;
    return $this->rightsData[$id];
  }

  function getExtendedData($id="")
  {
    if (!isset($this->extendedData)) $this->extendedData=array();
    if ($id=="") return $this->extendedData;
    if (!isset($this->extendedData[$id])) return FALSE;
    return $this->extendedData[$id];
  }

  // gibt die Anzahl der Kommentare vom Type $id zurck (default = anzahl aller kommentare)
  function gotComments($id="")
  {
    if (!isset($this->commentObjects)) {$this->commentObjects = array();}
    if ($id=="") return count($this->commentObjects);
    $counter = 0;
    foreach ($this->commentObjects as $comment) {
      if ($comment->getType()==$id) $counter++;
    }
    return $counter;
  }

  // gibt Array mit allen Comment-Objekten von entsprechendem Typ zurck
  function getComments($typeid="")
  {
    if (!isset($this->commentObjects)) {$this->commentObjects = array();}

    if ($typeid!="")
    {
      $comArray = array();
      foreach ($this->commentObjects as $comment)
      {
        if ($comment->getType()==$typeid)
        {
          array_push($comArray, $comment);
        }
      }
      return $comArray;
    }

    return $this->commentObjects;
  }

  function getCommentByTimeID($timeID)
  {
    if ($this->gotComments())
    {
      foreach ($this->commentObjects as $obj)
      {
        if ($obj->getPublishedUnixtime()==$timeID)
        {
          return $obj;
        }
      }
    }
    return FALSE;
  }

  // gibt Array mit allen verfgbaren Formaten zurck
  function getAvailableFormats()
  {
    $formats = array();
    if ((isset($this->alternative_files)) && (count($this->alternative_files)>0) )
    {
      foreach ($this->alternative_files as $key => $value)
      {
        array_push($formats, strtoupper($key));
      }
    }
    else
    {
      if (!$this->isFileInternetlink())
      {
        // wenn kein eintrag in alternative_files, dann $file eintragen
        $this->alternative_files[$this->getFormatByFileending($this->file)]["NORM"] = $this->file;
        array_push($formats, strtoupper($this->getFormatByFileending($this->file)));
      }
    }
    return $formats;
  }

  // gibt Array mit alles verfgbaren Qualitten fr das gegebene Format zurck
  function getAvailableQualities($format)
  {
    $qualities = array();
    if (isset($this->alternative_files[$format]))
    {
      foreach ($this->alternative_files[$format] as $key => $value)
      {
        array_push($qualities, strtoupper($key));
      }
    }
    return $qualities;
  }

  // Alle zum Artikel gehrige Dateien listen (Audio plus Artwork)
  function getAllAppendantFiles()
  {
    $files = array();

    if ( (isset($this->alternative_files)) && (count($this->alternative_files)>0) )
    {
      // Alle Eintrge in alternative_files durchlaufen und hinzufgen
      foreach ($this->alternative_files as $format)
      {
        foreach ($format as $filename)
        {
          array_push($files, $filename);
        }
      }
    }
    else
    {
      // wenn kein eintrag in alternative_files, dann $file prfen
      if ( ($this->file!="") && (!$this->isFileInternetlink()) )
      {
        array_push($files, $this->file);
      }
    }

    if ( ($this->artwork!="") && (substr_count($this->artwork,"://")==0) )
    {
      array_push($files, $this->artwork);
    }

    return $files;
  }


  // ------------------------------------------------------------
  // Weitere Aktionen auf Objekt
  // ------------------------------------------------------------

  // Gibt TRUE zurck, wenn der Suchstring auf die Funktion zutrifft
  // Suche nur auch EINEM Suchwort
  function matchesSearchString($search)
  {
    if ($search == "") return FALSE;

    $isMatch = FALSE;

    if (stristr($this->title, 		 $search)) {$isMatch=TRUE;}
    if (stristr($this->description , $search)) {$isMatch=TRUE;}
    if (stristr($this->authorName, 	 $search)) {$isMatch=TRUE;}
    if (stristr($this->description , $search)) {$isMatch=TRUE;}
    if (stristr($this->authorName, 	 $search)) {$isMatch=TRUE;}
    if (stristr($this->authorComment,$search)) {$isMatch=TRUE;}
    if (stristr($this->category, 	 $search)) {$isMatch=TRUE;}

    foreach ($this->getExtendedData() as $data)
    {
      if (stristr($data, $search)) {$isMatch=TRUE;}
    }

    return $isMatch;
  }

  // das Kommentarobjekt wir entfernt, wenn der Quickdeletecode stimmt
  function tryQuickDeleteComment($commentID, $deleteCode)
  {
    if ($this->gotComments())
    {
      // Comment-Objekt suchen und Code prfen
      $codeOK = FALSE;
      if ($obj = $this->getCommentByTimeID($commentID))
      {
        if ($deleteCode == md5($obj->getInsideQuickdeleteCode().$GLOBALS["ADMIN_PASSWORD"]))
        {
          // nur lschen, wenn der Kommentar nicht lter als 3 Tage ist
          // schtzt vor Missbrauch des Quickdeletes
          if ( $obj->getPublishedUnixtime()>(time()-(3*24*60*60)) )
          {
            $codeOK = TRUE;
            print "<!$deleteCode==".md5($obj->getInsideQuickdeleteCode().$GLOBALS["ADMIN_PASSWORD"]).">";
          }
        }
      }

      // Wenn Code OK, dann Kommentar lschen
      if ($codeOK)
      {
        $this->deleteComment($commentID);
        return TRUE;
      }
      return FALSE;
    }
  }

  // gibt TRUE zurck, wenn zu diesem Artikel mehr als eine Mediendatei gehrt
  function isMoreThenOneFile()
  {
    if ((isset($this->alternative_files)) && (count($this->alternative_files)>1) )
    {
      return TRUE;
    }
    else
    {
      return FALSE;
    }
  }

  // gibt TRUE zurck, wenn die Mediendatei eine bloe Internetreferenz ist
  function isFileInternetlink()
  {
    if (substr_count($this->file,"://")==0) {
      return FALSE;
    } else {
      return TRUE;
    }
  }

/**
* Name: save($dir)
*
* saves the article objekt aa <id>.article file into the given folder
* return TRUE or FALSE;
*/
// speicher das Artikel-Objekt als article-Datei im angegebenen Verzeichnis
  function save($dir)
  {
    $data_str = serialize($this);
    if ($fp = fopen($dir."/".$this->id.".article","w"))
    {
      fwrite($fp,$data_str);
      fclose($fp);
      return TRUE;
    }
    return FALSE;
  }

/**
* Name: checkValid()
*
* check if the article objekt is valid,
* an article is valid with a ID,Unixtime and Title
* return TRUE;
*/
// Ein Artikel ist gltig (VALID) , wenn ID, UNIXTIME und ein TITEL gesetzt sind
  function checkValid()
  {
    if ($this->id == 0) 	  	{return FALSE;}
    if ($this->unixTime == 0) 	{return FALSE;}
    if ($this->title == "")	  	{return FALSE;}

    $this->objectState = "VALID";
    return TRUE;
  }

  // ein Artikel kann publiziert werden, wenn er valid ist und
  // mindestens einen MP3-File oder einen Internetlink auf einen MP3-File hlt
  function isReadyToPublish()
  {
    if (isset($GLOBALS['PUBLISHFORMATS']))
    {
      $validFormats = explode(";",$GLOBALS['PUBLISHFORMATS']);
    }
    else
    {
      $validFormats = array(); array_push($validFormats,"MP3");
    }

    if ( !$this->checkValid() ) 					 return FALSE;
    if ($this->isFileInternetlink())				 return TRUE;

    if ( !$this->isMoreThenOneFile() )
    {
      foreach ($validFormats as $vFormat)
      {
        if ( $this->getFormatByFileending($this->file)==trim(strtoupper($vFormat)) ) return TRUE;
      }
    }
    else
    {
      if (isset($this->alternative_files['MP3'])) {
        foreach ($this->alternative_files['MP3'] as $file)
        {
          foreach ($validFormats as $vFormat)
          {
            if ( $this->getFormatByFileending($file)==trim(strtoupper($vFormat)) ) return TRUE;
          }
        }
      }
    }
    return FALSE;
  }

/**
* Name: moreFiles()
*
* returns Array of further files be longing to this article or
* return FALSE;
*/
  // returns Array of further files belonging to this article or FALSE
  function moreFiles()
  {
    $moreFiles = Array();
    // get directory of article
    if ($dir = $this->whereis_article($this->getID()))
    {
      $orgFile = $this->getFile();
      $tmp = explode(".",$orgFile); $orgEnding = array_pop($tmp);
      $orgFilenameWithoutEnding = implode(".",$tmp);

      // Seach Directory for Files with different Endings
     // Search in PublicData
     $verzeichnis = opendir($dir);
      while ($datei = readdir($verzeichnis))
      {
        $tmp = explode(".",$datei); $ending = array_pop($tmp);
        $dateiWithoutEnding = implode(".",$tmp);
        if (($dateiWithoutEnding == $orgFilenameWithoutEnding)  && ($ending!=$orgEnding))
        {array_push($moreFiles,$datei);}
      }
      closedir($verzeichnis);
      if (count($moreFiles)==0) {return FALSE;}
      else {return $moreFiles;}
    }
    else
    {
      return FALSE;
    }
  }

  // ------------------------------------------------------------
  // Statische Funktionen
  // ------------------------------------------------------------

/**
* Name: loadArticleFromFile($file)
*
* cinstructs an article objekt from a <id>.article file
* returns unserialize($object_str) or FALSE
*/
// Erstellt ein article-Objekt aus einer article-Datei
  function loadArticleFromFile($file)
  {
    // prfen ob es eine article-Datei ist
    $teile = explode(".",$file);
    if (array_pop($teile) == "article")
    {
      if ((file_exists($file)) && ($fp = fopen($file,"r")))
      {
        $object_str = implode("",file($file));
        return unserialize($object_str);
      }
      else
      {
        return FALSE;
      }
    }
    else
    {
      return FALSE;
    }
  }

  /**
  * Name: convert_ArrayInf_2_ObjectInf($file,$newID)
  *
  * converts article file of old podblogger 0.2 version
  * $file  = old inf file
  * $newID = the ID for the new article
  */
  function convert_ArrayInf_2_ObjectInf($file,$newID)
  {
     // Wenn Datei existiert, den alten Array laden
     if (file_exists($file))
     {
        // --------- prfen, ob es sich um eine alte Version handelt
        $test_str = 'O:7:"article"';
        $inf = fopen($file,"r");
        $datstr = fread($inf,25);
        fclose($inf);

        if (strstr($datstr,$test_str) != FALSE)
        {
          // Die Datei schein bereits auf dem neuesten Stand zu sein
          //print "$file ist bereits ein objekt<br>";
          return FALSE;
        }

        // ----------- Array aus alter Dateistruktur laden und in neues Objekt bertragen
        $artikel = new article($newID);

        $infData = array();
        $inf = fopen($file,"r");

        // ZEIT
        $data = article::freadLine($inf);
        if (strlen($data)==0) {$data = "0";}
        $artikel->setUnixTime($data);

        // DATEINAME
        $data = article::freadLine($inf);
        $data = stripslashes($data);
        if (strlen($data)==0) {$data = "";}
        $artikel->setFile($data);

        // TITEL
        $data = article::freadLine($inf);
        $data = stripslashes($data);
        if (strlen($data)==0) {$data = "";}
        $artikel->setTitle($data);

        // BESCHREIBUNG
        $data = article::freadLine($inf);
        $data = stripslashes($data);
        $data = str_replace("\r\n","\n",$data);
        $data = str_replace("\n\r","\n",$data);
        $data = str_replace("\r\n","\n",$data);
        $data = str_replace("<br>","\n",$data);
        $data = strip_tags($data);
        if (strlen($data)==0) {$data = "";}
        $artikel->setDescription($data);

        // AUTOR_NAME
        $data = article::freadLine($inf);
        $data = stripslashes($data);
        if (strlen($data)==0) {$data = "";}
        $artikel->setAuthorName($data);

        // AUTOR_MAIL
        $data = article::freadLine($inf);
        $data = stripslashes($data);
        if (strlen($data)==0) {$data = "";}
        $artikel->setAuthorMail($data);

        // AUTOR_TELEFON
        $data = article::freadLine($inf);
        $data = stripslashes($data);
        if (strlen($data)==0) {$data = "";}
        $artikel->setAuthorXtra($data);

        // AUTOR BEMERKUNG
        $data = article::freadLine($inf);
        $data = stripslashes($data);
        if (strlen($data)==0) {$data = "";}
        $artikel->setAuthorComment($data);

        // KATEGORIE
        $data = article::freadLine($inf);
        $data = stripslashes($data);
        if (strlen($data)==0) {$data = "";}
        $artikel->setCategory($data);

        fclose($inf);

        // Wenn Objekt gltig ....
        if ($artikel->checkValid())
        {
          // Verzeichnis aus $file extrahieren
          $teile = explode("/",$file);
          array_pop($teile);
          $dir = implode("/",$teile);

          // Objekt speichern
          $artikel->save($dir);

          // Alte Inf-Datei umbennen bzw. lschen
          if (!rename($file,$file.".bak")) {@unlink($file);}
        }

        return TRUE;
     }
     else
     {
      //print "$file existiert nicht.<br>";
      return FALSE;
     }
  }

/**
* Name: createUniqueID()
*
* creates a Unique number
* return time().$randZahl;
*/
  function createUniqueID()
  {
    $randZahl = mt_rand(1000,9999);
    return time().$randZahl;
  }


/**
* Name: freadLine($dat)
*
* is needed for old INF files
* return $out;
*/
// Wird fr das lesen der alten inf-Datei bentigt
  function freadLine($dat)
  {
    $out = "";
    $lastChar = "";
    while((!feof($dat)) && ($lastChar!="\n"))
    {
      $out = $out.$lastChar;
      $lastChar = fgetc($dat);
    }
    return $out;
  }

/**
* Name: whereis_article($articleid)
*
* returns the DIRECTORYname where article is located or
* returns FALSE
*/
// returns the DIRECTORYname where article is located oder returns FALSE
  function whereis_article($articleid)
  {
     print "\n<!--START WHERE IS ARTICLE-->";
     // Search in Uploaddata
     $verzeichnis = opendir("uploadData");
      while ($datei = readdir($verzeichnis))
      { if ( $datei == $articleid.".article" ) return "uploadData";}
      closedir($verzeichnis);

     // Search in PublicData
     $verzeichnis = opendir("publicData");
      while ($datei = readdir($verzeichnis))
      {
        print "\n<!--$datei == $articleid-->";
        if ( $datei == $articleid.".article" ) return "publicData";
      }
      closedir($verzeichnis);
      return FALSE;
  }

  // ermittelt anhand der Dateiendung das Format der Datei
  function getFormatByFileending($filename)
  {
    $parts  = explode(".",$filename);
    $endung = array_pop($parts);
    $endung = strtoupper($endung);
    return $endung;
  }

  // gets the filename of an file from the publicData directory and
  // gives back the article ID the file belongs to
  function getArticleIdByFilename($filename)
  {
    $files = getFilesFromDirWithEnding("publicData","article");
    foreach ($files as $file)
    {
      $artikel = article::loadArticleFromFile("publicData/".$file);
      if (is_object($artikel))
      {
        foreach ($artikel->getAllAppendantFiles() as $artikelfile)
        {
          if ("publicData/".$artikelfile == $filename) {return $artikel->getID();}
        }
      }
      else
      {
        // print "NO OBJECT!!"; exit();
      }
    }
    return FALSE;
  }

}

?>