<?
    /*
     * MP3 class (reduced)
     * based on MP3 class rel. 0.1 by Ludovico Magnocavallo ludo@sumatrasolutions.com
     * License type: GNU GPL http://www.gnu.org/copyleft/gpl.html
     *
     * Sample Usage:
     * <?
     * require("mp3Info.class.php");
     * $info = new mp3Info("audio_default.mp3");
     * print "Lnge: ".$info->getLength()."\n";
     * print "Bitrate: ".$info->getBitrate()."\n";
     * print "Samplerate: ".$info->getSamplerate()."\n";
     * print "Mono: ".$info->isMono()."\n";
     * ?>
     *
     */
    class mp3Info {

        var $info_bitrates = array(
            1    =>    array(
                1    =>    array( 0 => 0, 16 => 32, 32 => 64, 48 => 96, 64 => 128, 80 => 160, 96 => 192, 112 => 224, 128 => 256, 144 => 288, 160 => 320, 176 => 352, 192 => 384, 208 => 416, 224 => 448, 240 => false),
                2    =>    array( 0 => 0, 16 => 32, 32 => 48, 48 => 56, 64 =>  64, 80 =>  80, 96 =>  96, 112 => 112, 128 => 128, 144 => 160, 160 => 192, 176 => 224, 192 => 256, 208 => 320, 224 => 384, 240 => false),
                3    =>    array( 0 => 0, 16 => 32, 32 => 40, 48 => 48, 64 =>  56, 80 =>  64, 96 =>  80, 112 =>  96, 128 => 112, 144 => 128, 160 => 160, 176 => 192, 192 => 224, 208 => 256, 224 => 320, 240 => false)
            ),
            2    =>    array(
                1    =>    array( 0 => 0, 16 => 32, 32 => 48, 48 => 56, 64 =>  64, 80 => 80, 96 => 96, 112 => 112, 128 => 128, 144 => 144, 160 => 160, 176 => 176, 192 => 192, 208 => 224, 224 => 256, 240 => false),
                2    =>    array( 0 => 0, 16 =>  8, 32 => 16, 48 => 24, 64 =>  32, 80 => 40, 96 => 48, 112 =>  56, 128 =>  64, 144 =>  80, 160 =>  96, 176 => 112, 192 => 128, 208 => 144, 224 => 160, 240 => false),
                3    =>    array( 0 => 0, 16 =>  8, 32 => 16, 48 => 24, 64 =>  32, 80 => 40, 96 => 48, 112 =>  56, 128 =>  64, 144 =>  80, 160 =>  96, 176 => 112, 192 => 128, 208 => 144, 224 => 160, 240 => false)
            )
        );
        var $info_versions = array(0 => "reserved", 1 => "MPEG Version 1", 2 => "MPEG Version 2", 2.5 => "MPEG Version 2.5");
        var $info_layers = array("reserved", "Layer I", "Layer II", "Layer III");
        var $info_sampling_rates = array(
            0        =>    array(0 => false, 4 => false, 8 => false, 12 => false),
            1        =>    array(0 => "44100 Hz", 4 => "48000 Hz", 8 => "32000 Hz", 12 => false),
            2        =>    array(0 => "22050 Hz", 4 => "24000 Hz", 8 => "16000 Hz", 12 => false),
            2.5    =>    array(0 => "11025 Hz", 4 => "12000 Hz", 8 => "8000 Hz", 12 => false)
        );
        var $info_channel_modes = array(0 => "stereo", 64 => "joint stereo", 128 => "dual channel", 192 => "single channel");
        var $file = "";
        var $fh = false;
        var $error = false;
        var $url = "";
        var $info = array();

		// ######### PUBLIC METHODS ###########################

        function mp3Info($file, $exitonerror=true) {
            if (file_exists($file)) {
                $this->file = $file;
                $this->fh = fopen($this->file,"r");
                global $HTTP_HOST, $PHP_SELF;
                $this->url = "http://$HTTP_HOST/$PHP_SELF";
                $this->get_info();
            } else {
                $this->error = "No such file";
            }
        }

        function getLength() // HH:MM:SS
        {
          return $this->info["length"];
        }

        function getBitrate()
        {
          return $this->info["bitrate"];
        }

        function getSamplerate()
        {
          return $this->info["sampling_rate"];
        }

        function isMono()
        {
          if ($this->info["channel_mode"] == "single channel") 	{ return TRUE; }
          else 													{ return FALSE; }
        }

		// ######### PRIVATE  METHODS ###########################

        function exitonerror() {
            echo($this->error);
            exit;
        }

        // get_info() helper methods
        function calculate_length($id3v2_tagsize = 0) {
           if ($this->info["bitrate"]>0) {
            $length = floor(($this->info["filesize"] - $id3v2_tagsize) / $this->info["bitrate"] * 0.008);
            $h   = floor($length / 3600); $length = $length - ($h*3600);
            $h   = strlen($h) == 1 ? "0$h" : $h;
            $min = floor($length / 60);
            $min = strlen($min) == 1 ? "0$min" : $min;
            $sec = $length % 60;
            $sec = strlen($sec) == 1 ? "0$sec" : $sec;
            return("$h:$min:$sec");                 
           }  else {
            return "00:10:00";
           }

        }

        function get_info() {
            $second = $this->synchronize();
            $third = ord(fread($this->fh, 1));
            $fourth = ord(fread($this->fh, 1));
            $this->info["version_id"] = ($second & 16) > 0 ? ( ($second & 8) > 0 ? 1 : 2 ) : ( ($second & 8) > 0 ? 0 : 2.5 );
            $this->info["version"] = $this->info_versions[ $this->info["version_id"] ];
            $this->info["layer_id"] = ($second & 4) > 0 ? ( ($second & 2) > 0 ? 1 : 2 ) : ( ($second & 2) > 0 ? 3 : 0 );     ;
            $this->info["layer"] = $this->info_layers[ $this->info["layer_id"] ];
            $this->info["protection"] = ($second & 1) > 0 ? "no CRC" : "CRC";
            $this->info["bitrate"] = $this->info_bitrates[ $this->info["version_id"] ][ $this->info["layer_id"] ][ ($third & 240) ];
            $this->info["sampling_rate"] = $this->info_sampling_rates[ $this->info["version_id"] ][ ($third & 12)];
            $this->info["padding"] = ($third & 2) > 0 ? "on" : "off";
            $this->info["private"] = ($third & 1) > 0 ? "on" : "off";
            $this->info["channel_mode"] = $this->info_channel_modes[$fourth & 192];
            $this->info["copyright"] = ($fourth & 8) > 0 ? "on" : "off";
            $this->info["original"] = ($fourth & 4) > 0 ? "on" : "off";
            $this->info["filesize"] = filesize($this->file);
            $this->info["length"] = $this->calculate_length();
        }

        function synchronize() {
            $finished = false;
            rewind($this->fh);
            while (!$finished) {
                $skip = ord(fread($this->fh, 1));
                while ($skip != 255 && !feof($this->fh)) {
                    $skip = ord(fread($this->fh, 1));
                }
                if (feof($this->fh)) {
                    $this->error("no info header found");
                    if ($exitonerror) $this->exitonerror();
                }
                $store = ord(fread($this->fh, 1));
                if ($store >= 225) {
                    $finished = true;
                } else if (feof($this->fh)) {
                    $this->error("no info header found");
                    if ($exitonerror) $this->exitonerror();
                }
            }
            return($store);
        }

        function close() {
            @fclose($this->fh);
        }
    }
?>